# Copyright 2017 SrMouraSilva
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import uuid


class Indexable(object):
    def __init__(self):
        self._uuid = str(uuid.uuid4())

    @property
    def uuid(self):
        """
        :return string: Unique identifier used for saves
        """
        return self._uuid

    @property
    def simple_identifier(self):
        """
        Simple identifier for index file human comprehension and reordering
        :return string:
        """
        pass
